"""
Module for interacting with the 4chan JSON API
"""
import json
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


class InvalidApiResponse(Exception):
    pass


class ApiRequestException(Exception):
    pass


class ChanAPI:
    """
    Wrapper class for interacting with the 4chan JSON API.

    Args:
        headers (dict, optional): Dictionary containing customer headers to be sent with each request.
        proxies (dict, optional): Dictionary containing proxy settings to be used for requests.

    Attributes:
        base_url (str): The base URL of the API.
        session (requests.Session): Persistent HTTP session.
    """
    def __init__(self, headers=None, proxies=None):
        self.base_url = 'https://a.4cdn.org'
        self.session = requests.Session()

        retries = Retry(total=3, backoff_factor=1, status_forcelist=[429, 500, 502, 503, 504])
        self.session.mount('http://', HTTPAdapter(max_retries=retries))
        self.session.mount('https://', HTTPAdapter(max_retries=retries))

        if headers:
            self.session.headers.update(headers)
        if proxies:
            self.session.proxies.update(proxies)

    def __get(self, endpoint, params=None):
        url = self.base_url + endpoint

        try:
            response = self.session.get(url, params=params)
            response.raise_for_status()
        except requests.RequestException as e:
            raise ApiRequestException(f'Failed to execute GET request: {str(e)}')

        try:
            return response.json()
        except json.JSONDecodeError:
            raise InvalidApiResponse('Invalid JSON received')

    def get_boards(self):
        endpoint = '/boards.json'
        response = self.__get(endpoint)
        return response['boards']

    def get_catalog(self, board):
        endpoint = f'/{board}/catalog.json'
        response = self.__get(endpoint)
        return response

    def get_threads(self, board):
        endpoint = f'/{board}/threads.json'
        response = self.__get(endpoint)
        return response

    def get_thread(self, board, no):
        endpoint = f'/{board}/thread/{no}.json'
        response = self.__get(endpoint)
        return response
